#include <fcntl.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>

struct timeval SerialTimeOut = {1, 0};

int openSerialPort(char *dev_name)
{
  int fd;
  struct termios flags;

  if ((fd = open(dev_name, O_RDWR | O_NONBLOCK | O_NOCTTY)) < 0) {
    return -1;
  }
  if (tcgetattr(fd, &flags)) {
    close(fd);
    return -1;
  }
  cfmakeraw(&flags);
  flags.c_cflag |= CLOCAL;

  cfsetispeed(&flags, B9600);
  cfsetospeed(&flags, B9600);

  /* stopBits = 1 */
  flags.c_cflag &= ~CSTOPB;

  /* parity None */
  flags.c_cflag &= ~(PARENB | PARODD);

  /* dataBits = 8 */
  flags.c_cflag &= ~CSIZE;
  flags.c_cflag |= CS8;

  /* no flow control */
  flags.c_iflag &= ~(IXON | IXOFF | IXANY);
#if defined(CRTSCTS)
  flags.c_cflag &= ~CRTSCTS;
#endif

  if (tcsetattr(fd, TCSANOW, &flags)) {
    close(fd);
    return -1;
  }

  return fd;
}

int closeSerialPort(int fd)
{
  if (close(fd)) {
    return -1;
  }
  return 0;
}

int readSerialPortInto(int fd, int count, char *startPtr)
{
  ssize_t bytesRead = 0;
  void *buffer = (void *)startPtr;

  bytesRead = read(fd, buffer, (size_t)count);
  if (bytesRead == (ssize_t)-1) {
    if (errno == EAGAIN) {
      bytesRead = 0;
    } else {
      return 0;
    }
  }
  return bytesRead;
}

int writeSerialPortFrom(int fd, int count, char *startPtr)
{
  int bytesWritten;
  void *buffer = (void *)startPtr;

  bytesWritten = write(fd, buffer, (size_t)count);
  if (bytesWritten == (ssize_t)-1) {
    return 0;
  }
  tcdrain(fd);
  return bytesWritten;
}

int readSerialPortUpto(int fd, char *startPtr, int upto)
{
  int bytesRead;
  int totalBytes = 0;
  struct timeval tv, tx;

  gettimeofday(&tv, NULL);
  tv.tv_sec += SerialTimeOut.tv_sec;
  tv.tv_usec += SerialTimeOut.tv_usec;
  if (tv.tv_usec >= 1000000) {
    tv.tv_sec++;
    tv.tv_usec -= 1000000;
  }

  while (totalBytes < upto) {
    bytesRead = readSerialPortInto(fd, upto - totalBytes, startPtr);
    startPtr += bytesRead;
    totalBytes += bytesRead;

    gettimeofday(&tx, NULL);
    if (tx.tv_sec == tv.tv_sec) {
      if (tx.tv_usec >= tv.tv_usec) {
	break;
      }
    } else {
      if (tx.tv_sec > tv.tv_sec) {
	break;
      }
    }
  }
  return totalBytes;
}

void clearInputBuffer(int fd)
{
  char buf;
  while(readSerialPortInto(fd, 1, &buf) == 1)
    ;
}
